<!---
	COMPONENT: ContactListner
	AUTHOR: Matt Woodward (mpwoodward@gmail.com)
	DATE: 7/17/2005
	PURPOSE: ContactListener for Mach-II Contact Manager sample application
				* based on ContactManager CFC from original Mach-II Contact Manager sample application
				* combined functionality from ContactManager and RecentContacts CFCs in original application 
					into a single CFC for the updated version of the application
	CHANGE HISTORY:
		* 7/17/2005: component created (Matt Woodward)
--->

<cfcomponent displayname="ContactListener" output="false" extends="MachII.framework.Listener" 
		hint="ContactListener for the Mach-II Contact Manager sample application">
	<cffunction name="configure" access="public" output="false" returntype="void" 
			hint="Configures this listener as part of the Mach-II framework">
		<!--- create an empty struct to hold all the contact objects for the application --->
		<cfset variables.contacts = StructNew() />
		
		<!--- create an empty array to hold recent contacts for the application --->
		<cfset variables.recentContacts = ArrayNew(1) />
	</cffunction>
	
	<!--- CONTACT METHODS --->
	<!--- get all contacts - returns the struct containing all the contact objects --->
	<cffunction name="getAllContacts" access="public" output="false" returntype="struct" 
			hint="Returns the struct containing all the contact objects">
		<cfreturn variables.contacts />
	</cffunction>
	
	<!--- get contact - returns a specific contact object --->
	<cffunction name="getContact" access="public" output="false" returntype="ContactManager.model.Contact" 
			hint="Returns a specific contact object based on the id passed to this function">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfif StructKeyExists(variables.contacts, arguments.event.getArg("id"))>
			<cfreturn variables.contacts[arguments.event.getArg("id")] />
			<cfelse>
				<cfthrow message="Contact with id #arguments.event.getarg('id')# does not exist." />
		</cfif>
	</cffunction>
	
	<!--- create contact using simple form values as event arguments --->
	<cffunction name="createContactFromEventArgs" access="public" output="false" returntype="void" 
			hint="Creates a contact using the form values directly and announces the next event">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfscript>
			// struct to hold new event args
			var newEventArgs = StructNew();
			
			// create id for this contact
			var id = CreateUUID();
			
			// instantiate contact object
			var contact = CreateObject("component", 
							"ContactManager.model.Contact").init(id, arguments.event.getArg("firstName"), 
																arguments.event.getArg("lastName"), 
																arguments.event.getArg("street"), 
																arguments.event.getArg("city"), 
																arguments.event.getArg("state"), 
																arguments.event.getArg("zip"));
			
			// put contact object in contacts struct
			variables.contacts[contact.getId()] = contact;
			
			// put contact in new event args so it's accessible in the next event
			newEventArgs["contact"] = contact;
			
			// announce next event
			announceEvent("contactCreated", newEventArgs);
		</cfscript>
	</cffunction>
	
	<!--- create contact using event bean --->
	<cffunction name="createContactFromEventBean" access="public" output="false" returntype="void" 
			hint="Creates a contact using the Contact object as an event bean">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfscript>
			// struct to hold new event args
			var newEventArgs = StructNew();
			
			// create id for this contact
			var id = CreateUUID();
			
			// set the id in the event bean, which comes through as the arg "contact" in the event
			arguments.event.getArg("contact").setId(id);
			
			// since we already have the populated event bean, put it in the contacts struct
			variables.contacts[arguments.event.getArg("contact").getId()] = arguments.event.getArg("contact");
			
			// put contact in new event args so it's accessible in the next event
			newEventArgs["contact"] = arguments.event.getArg("contact");
			
			// announce next event
			announceEvent("contactCreated", newEventArgs);
		</cfscript>
	</cffunction>
	
	<!--- update contact using simple form values as event arguments (not used in app; uses event bean
		  	for updates by default) --->
	<cffunction name="updateContactFromEventArgs" access="public" output="false" returntype="void" 
			hint="Updates a contact using the form values directly and announces the next event">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfscript>
			// struct for new event args
			var newEventArgs = StructNew();
			
			// set contact var for thread safety purposes
			var contact = "";
			
			// message to display in next event if necessary
			var message = "";
			
			// if the contact exists, update it
			if (StructKeyExists(variables.contacts, arguments.event.getArg("id"))) {
				// get the contact from the struct
				contact = getContact(arguments.event.getArg("id"));
				
				// update values in contact object
				contact.setFirstName(arguments.event.getArg("firstName"));
				contact.setLastName(arguments.event.getArg("lastName"));
				contact.setStreet(arguments.event.getArg("street"));
				contact.setCity(arguments.event.getArg("city"));
				contact.setState(arguments.event.getArg("state"));
				contact.setZip(arguments.event.getArg("zip"));
			} else {
				// contact doesn't exist
				message = "Contact with id #arguments.event.getArg('id')# doesn't exist.";
			}
			
			// set new event args
			newEventArgs["contact"] = contact;
			newEventArgs["message"] = message;
			
			// announce next event
			announceEvent("contactUpdated", newEventArgs);
		</cfscript>
	</cffunction>
	
	<!--- update contact using event bean --->
	<cffunction name="updateContact" access="public" output="false" returntype="void" 
			hint="Updates a contact using the Contact object as an event bean">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfscript>
			// struct for new event args
			var newEventArgs = StructNew();
			
			// set contact var for thread safety purposes
			var contact = "";
			
			// message to display in next event if necessary
			var message = "";
			
			// variables for array loop
			var i = 0;
			var curContact = "";
			var addToRecent = false;
			var recentContactIdList = "";
			
			// if the contact exists, update it
			if (StructKeyExists(variables.contacts, arguments.event.getArg("contact").getId())) {
				// set contact in struct to event bean
				variables.contacts[arguments.event.getArg("contact").getId()] = arguments.event.getArg("contact");
				
				// get a list of IDs from the recent contacts array
				for (i = 1; i LTE ArrayLen(variables.recentContacts); i = i + 1) {
					curContact = variables.recentContacts[i];
					recentContactIdList = ListAppend(recentContactIdList, curContact.getId(), ",");
				}
				
				// set contact in recent contacts array to event bean
				if (ListLen(recentContactIdList) GT 0) {
					if (ListFind(recentContactIdList, arguments.event.getArg("contact").getId(), ",")) {
						for (i = 1; i LTE ArrayLen(variables.recentContacts); i = i + 1) {
							curContact = variables.recentContacts[i];
							if (curContact.getId() EQ arguments.event.getArg("contact").getId()) {
								variables.recentContacts[i] = arguments.event.getArg("contact");
							}
						}
					} else {
						addToRecent = true;
					}
				} else {
					// no recent contacts, but need to add this one
					addToRecent = true;
				}
			} else {
				// contact doesn't exist
				message = "Contact with id #arguments.event.getArg('contact').getId()# doesn't exist.";
			}
			
			if (addToRecent) {
				addRecentContact(arguments.event);
			}
			
			// set new event args
			newEventArgs["contact"] = contact;
			newEventArgs["message"] = message;
			
			// announce next event
			announceEvent("contactUpdated", newEventArgs);
		</cfscript>
	</cffunction>
	
	<!--- delete contact --->
	<cffunction name="deleteContact" access="public" output="false" returntype="void" 
			hint="Deletes a contact and announces the next event">
		<cfargument name="event" type="MachII.framework.event" required="true" />
		
		<cfscript>
			// struct for new event args
			var newEventArgs = StructNew();
			
			// message to display in next event
			var message = "Contact deleted successfully.";
			
			// if the contact exists, delete it
			if (StructKeyExists(variables.contacts, arguments.event.getArg("id"))) {
				StructDelete(variables.contacts, arguments.event.getArg("id"), false);
			} else {
				message = "Contact with id #arguments.event.getArg('id')# doesn't exist.";
			}
			
			// set new event args
			newEventArgs["message"] = message;
			newEventArgs["id"] = arguments.event.getArg("id");
			
			// announce next event
			announceEvent("contactDeleted", newEventArgs);
		</cfscript>
	</cffunction>
	
	<!--- RECENT CONTACT METHODS --->
	<cffunction name="addRecentContact" access="public" output="false" returntype="void" 
			hint="Adds a contact object to the recentContacts array">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfset ArrayPrepend(variables.recentContacts, arguments.event.getArg("contact")) />
	</cffunction>
	
	<cffunction name="getRecentContacts" access="public" output="false" returntype="array" 
			hint="Returns an array of recent contacts">
		<cfreturn variables.recentContacts />
	</cffunction>
	
	<cffunction name="deleteRecentContact" access="public" output="false" returntype="void" 
			hint="Deletes a contact object from the recentContacts array">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<cfscript>
			// set defaults
			var i = 0;
			var contactPos = 0;
			var currentContact = "";
			
			// loop through to find item position
			if (ArrayLen(variables.recentContacts) GT 0) {
				for (i = 1; i LTE ArrayLen(variables.recentContacts); i = i + 1) {
					currentContact = variables.recentContacts[i];
					if (currentContact.getId() EQ arguments.event.getArg("id")) {
						contactPos = i;
					}
				}
			}
			
			// if the items was found, delete it
			if (contactPos NEQ 0) {
				ArrayDeleteAt(variables.recentContacts, contactPos);
			}
		</cfscript>
	</cffunction>
</cfcomponent>